@@ -4,6 +4,7 @@ from contextlib import contextmanager
 
 from joblib.parallel import DEFAULT_BACKEND
 
+
 default_options = {
     "reproducibility": {"random_seed": 123},
     "db": {
@@ -21,7 +22,7 @@ default_options = {
         "cache": {
             "disable": True,
             "backend": "joblib.Memory",
-            "location": ".joblib.memory",  # f"{tempfile.gettempdir()}{os.sep}mltraq.memory",
+            "location": ".joblib.memory",
         },
     },
     "tqdm": {"disable": False, "delay": 1},
@@ -45,6 +46,18 @@ default_options = {
 
 
 class Options:
+    """Handle options (preferences) for the package.
+
+    Raises:
+        RuntimeError: _description_
+
+    Returns:
+        _type_: _description_
+
+    Yields:
+        _type_: _description_
+    """
+
     _instance = None
 
     def __init__(self):
@@ -93,4 +106,6 @@ class Options:
             self.options = orig_options
 
 
+# This object handles the options for the package, process-wide.
+# To change locally the preferences, use option_context.
 options = Options.instance()
