@@ -11,10 +11,28 @@ from mltraq.options import options
 from mltraq.utils.dicts import ObjectDictionary
 from mltraq.utils.frames import json_normalize
 
-TYPE_KEY = "mltraq_type"
+# Keyword used to identify serialized values and the version of the serialization format.
+TYPE_KEY = "mltraq-type-1.0"
+
+# Introduced with Python 3.0, https://peps.python.org/pep-3154/
+# Unpickling might fail: on different architectures, different python version, in case of missing packages.
+PICKLE_DEFAULT_PROTOCOL = 4
+
+# Types that are handled by serialization.
+SERIALIZED_TYPES = [pd.DataFrame, pd.Series, np.ndarray, uuid.UUID, dict, list]
 
 
 def compress(data: bytes, enable_compression=None) -> bytes:
+    """Compress a sequence of bytes.
+
+    Args:
+        data (bytes): Data to compress.
+        enable_compression (_type_, optional): Force compression (enable/disable)
+            regardless the default. Defaults to None.
+
+    Returns:
+        bytes: Compressed data.
+    """
     if enable_compression is None:
         enable_compression = options.get("serialization.enable_compression")
 
@@ -25,6 +43,15 @@ def compress(data: bytes, enable_compression=None) -> bytes:
 
 
 def decompress(data: bytes) -> bytes:
+    """Decompress the data. It works also with uncompressed data: if zlib fails,
+        it returns the input data.
+
+    Args:
+        data (bytes): Data to decompress.
+
+    Returns:
+        bytes: Decompressed data.
+    """
     if isinstance(data, memoryview):
         data = data.tobytes()
 
@@ -36,11 +63,6 @@ def decompress(data: bytes) -> bytes:
     return data
 
 
-# Introduced with Python 3.0, https://peps.python.org/pep-3154/
-# Unpickling might fail: on different architectures, different python version, in case of missing packages.
-PICKLE_DEFAULT_PROTOCOL = 4
-
-
 def pickle_dumps(obj: object) -> bytes:
     """It returns the compressed (zlib) pickled object. The Pickle DEFAULT_PROTOCOL is used
     for maximum compatibility.
@@ -90,38 +112,50 @@ def pickle_size(obj: object, unit: str = "b") -> int:
         return None
 
 
-serialized_types = [pd.DataFrame, pd.Series, np.ndarray, uuid.UUID, dict, list]
-
-
 class JSONEncoder(json.JSONEncoder):
     def default(self, obj):
-        if isinstance(obj, pd.DataFrame):
-            # print(obj.columns)
-            # if "timestamp" in obj.columns:
-            #    print(obj.x)
-            dtypes = {col: str(obj[col].dtype) for col in obj.columns}
+        """Return the dictionary that represents the serialized version of an object
 
-            # handle timestamps serialization
+        Args:
+            obj (_type_): Object to serialize
+
+        Returns:
+            _type_: dict containing the serialized object
+        """
+        if isinstance(obj, pd.DataFrame) or isinstance(obj, pd.Series):
+            # Since, in presence of timestamps in either columns or index, we're going to change the data,
+            # let's work on a copy of the dataframe to not modify the original dataframe.
+            # If it's a series, we just convert it to a dataframe, implicitly obtaining a copy of the data.
+            if isinstance(obj, pd.Series):
+                obj = obj.to_frame()
+                obj_type = "pandas.Series"
+            else:
+                obj = obj.copy()
+                obj_type = "pandas.DataFrame"
+
+            dtypes = {col: str(obj[col].dtype) for col in obj.columns}
+            dtype_index = str(obj.index.dtype)
+
+            # handle timestamps serialization in columns
             cols_datetime64ns = [col for col in obj.columns if obj[col].dtype == "datetime64[ns]"]
             if len(cols_datetime64ns) > 0:
-                # some columns contain timestamps, let's convert them to strings on a frame copy
-                obj = obj.copy()
+                # some columns contain timestamps, let's convert them to strings
                 for col in cols_datetime64ns:
                     obj[col] = obj[col].astype(int)
 
+            # Timestamps in the index have a dtype "<M8[ns]" instead of "datetime64[ns]" (not sure why
+            # it's nost simply "datetime64[ns]" as for columns).  This is why we catch multiple dtype
+            # types that might be associated to timestamps.
+            if dtype_index in ["<M8[ns]", ">M8[ns]", "datetime64[ns]"]:
+                obj.index = obj.index.astype(int)
+
             return {
-                TYPE_KEY: "pandas.DataFrame",
+                TYPE_KEY: obj_type,
+                "dtype_index": dtype_index,
                 "dtypes": dtypes,
                 "data": obj.to_dict(orient="list"),
                 "index": obj.index.tolist(),
             }
-        elif isinstance(obj, pd.Series):
-            return {
-                TYPE_KEY: "pandas.Series",
-                "dtype": str(obj.dtype),
-                "data": obj.tolist(),
-                "index": obj.index.tolist(),
-            }
         elif isinstance(obj, np.ndarray):
             return {TYPE_KEY: "numpy.ndarray", "data": obj.tolist()}
         elif isinstance(obj, uuid.UUID):
@@ -131,6 +165,16 @@ class JSONEncoder(json.JSONEncoder):
 
 
 def serialize(obj: object, enable_compression=None) -> bytes:
+    """Serialize an object
+
+    Args:
+        obj (object): Object to serialize
+        enable_compression (_type_, optional): If not None, enable/disable compression.
+            If None, consider default preference. Defaults to None.
+
+    Returns:
+        bytes: Serialized object.
+    """
     if enable_compression is None:
         enable_compression = options.get("serialization.enable_compression")
 
@@ -138,19 +182,28 @@ def serialize(obj: object, enable_compression=None) -> bytes:
 
 
 def deserialize(obj: bytes) -> object:
+    """Deserialize an object
+
+    Args:
+        obj (bytes): Object to deserialize.
+
+    Returns:
+        object: Unserialized object.
+    """
+
     def f(v):
         if isinstance(v, dict) and TYPE_KEY in v:
             # This ia a value to deserialize
-            if v[TYPE_KEY] == "pandas.DataFrame":
+            if v[TYPE_KEY] in ["pandas.DataFrame", "pandas.Series"]:
                 df = pd.DataFrame.from_dict(v["data"], orient="columns")
-                df.index = pd.Index(v["index"])
+                df.index = pd.Index(v["index"]).astype(v["dtype_index"])
                 for col, dtype in v["dtypes"].items():
                     df[col] = df[col].astype(dtype)
-                return df
-            # elif v["__type"] == "pandas.Timestamp":
-            #    return pd.Series(v["data"], index=v["index"]).astype(pd.Timestamp, unit="ms")
-            elif v[TYPE_KEY] == "pandas.Series":
-                return pd.Series(v["data"], index=v["index"]).astype(v["dtype"])
+                if v[TYPE_KEY] == "pandas.DataFrame":
+                    return df
+                else:
+                    # Return first column, a series
+                    return df[df.columns[0]]
             elif v[TYPE_KEY] == "numpy.ndarray":
                 return np.asarray(v["data"])
             elif v[TYPE_KEY] == "uuid.UUID":
@@ -172,6 +225,16 @@ def deserialize(obj: bytes) -> object:
 
 
 def serialize_df(df: pd.DataFrame, ignore_columns: list = None, enable_compression=None):
+    """Serialize some of the columns of the dataframe, making it ready for database storage.
+
+    Args:
+        df (pd.DataFrame): Dataframe to process.
+        ignore_columns (list, optional): Columns to ignore. Defaults to None.
+        enable_compression (_type_, optional): If not None, force enable/disable of compression. Defaults to None.
+
+    Returns:
+        _type_: _description_
+    """
     if enable_compression is None:
         enable_compression = options.get("serialization.enable_compression")
 
@@ -180,7 +243,7 @@ def serialize_df(df: pd.DataFrame, ignore_columns: list = None, enable_compressi
     # Identify columns to serialize
     serialized_cols = []
     for col_name in consider_columns:
-        for serialized_type in serialized_types:
+        for serialized_type in SERIALIZED_TYPES:
             # We assume that the types in the first row are the same of the other rows.
             if isinstance(df[col_name].iloc[0], serialized_type):
                 serialized_cols.append(col_name)
