@@ -19,6 +19,16 @@ def reorder_columns(df: pd.DataFrame, ordered_columns: List[str]) -> List[str]:
 
 
 def json_normalize(data, max_level=0, sep="."):
+    """Convert JSON object to dataframe.
+
+    Args:
+        data (_type_): JSON object
+        max_level (int, optional): Max level to normalize. Defaults to 0.
+        sep (str, optional): Separator for nested values. Defaults to ".".
+
+    Returns:
+        _type_: _description_
+    """
     df = pd.json_normalize(
         data,
         max_level=max_level,
